﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

namespace FlipXYVideo
{
    public partial class MainPage : PhoneApplicationPage
    {
        int numRowsCols = 4;
        CaptureSource captureSource;
        VideoBrush videoBrush;
        
        public MainPage()
        {
            InitializeComponent();
            fewerButton = this.ApplicationBar.Buttons[0] as ApplicationBarIconButton;
            moreButton = this.ApplicationBar.Buttons[1] as ApplicationBarIconButton;

            captureSource = new CaptureSource
            {
                VideoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice()
            };
            videoBrush = new VideoBrush();
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            if (this.State.ContainsKey("numRowsCols"))
            {
                numRowsCols = (int)this.State["numRowsCols"];
                this.State.Remove("numRowsCols");
            }
            CreateRowsAndColumns();

            if (captureSource != null && CaptureDeviceConfiguration.AllowedDeviceAccess)
            {
                videoBrush.SetSource(captureSource);
                captureSource.Start();
                startButton.Visibility = Visibility.Collapsed;
            }

            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            this.State.Add("numRowsCols", numRowsCols);

            if (captureSource != null && captureSource.State == CaptureState.Started)
            {
                captureSource.Stop();
                startButton.Visibility = Visibility.Visible;
            }

            base.OnNavigatedFrom(args);
        }

        void OnStartButtonClick(object sender, RoutedEventArgs args)
        {
            if (captureSource != null && 
                (CaptureDeviceConfiguration.AllowedDeviceAccess ||
                 CaptureDeviceConfiguration.RequestDeviceAccess()))
            {
                videoBrush.SetSource(captureSource);
                captureSource.Start();
                startButton.Visibility = Visibility.Collapsed;
            }
        }

        void OnFewerButtonClick(object sender, EventArgs args)
        {
            numRowsCols--;
            CreateRowsAndColumns();
        }

        void OnMoreButtonClick(object sender, EventArgs args)
        {
            numRowsCols++;
            CreateRowsAndColumns();
        }

        void CreateRowsAndColumns()
        {
            videoPanel.Children.Clear();
            videoPanel.RowDefinitions.Clear();
            videoPanel.ColumnDefinitions.Clear();

            for (int row = 0; row < numRowsCols; row++)
                videoPanel.RowDefinitions.Add(new RowDefinition 
                { 
                    Height = new GridLength(1, GridUnitType.Star) 
                });

            for (int col = 0; col < numRowsCols; col++)
                videoPanel.ColumnDefinitions.Add(new ColumnDefinition 
                { 
                    Width = new GridLength(1, GridUnitType.Star) 
                });

            for (int row = 0; row < numRowsCols; row++)
                for (int col = 0; col < numRowsCols; col++)
                {
                    Rectangle rect = new Rectangle
                    {
                        Fill = videoBrush,
                        RenderTransformOrigin = new Point(0.5, 0.5),
                        RenderTransform = new ScaleTransform
                        {
                            ScaleX = 1 - 2 * (col % 2),
                            ScaleY = 1 - 2 * (row % 2),
                        },
                    };
                    Grid.SetRow(rect, row);
                    Grid.SetColumn(rect, col);
                    videoPanel.Children.Add(rect);
                }
            fewerButton.IsEnabled = numRowsCols > 1;
        }
    }
}