﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework.Media;

namespace Kaleidovideo
{
    public partial class MainPage : PhoneApplicationPage
    {
        CaptureSource captureSource;
 
        public MainPage()
        {
            InitializeComponent();

            VideoCaptureDevice videoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice();
            videoCaptureDevice.DesiredFormat = new VideoFormat(PixelFormatType.Format32bppArgb, 640, 480, 30);
            captureSource = new CaptureSource { VideoCaptureDevice = videoCaptureDevice };

            WriteableBitmap bitmap = new WriteableBitmap(105, 480);
            ImageBrush imgBrush = new ImageBrush { ImageSource = bitmap };
            rect1.Fill = imgBrush;
            rect2.Fill = imgBrush;
            rect3.Fill = imgBrush;
            rect4.Fill = imgBrush;
            rect5.Fill = imgBrush;
            rect6.Fill = imgBrush;

            KaleidoscopeVideoSink sink = new KaleidoscopeVideoSink(bitmap)
            {
                AllocationMode = SampleAllocationMode.ReuseBuffer,
                CaptureSource = captureSource
            };
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            if (captureSource != null && CaptureDeviceConfiguration.AllowedDeviceAccess)
            {
                captureSource.Start();
                startButton.Visibility = Visibility.Collapsed;
            }

            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            if (captureSource != null && captureSource.State == CaptureState.Started)
            {
                captureSource.Stop();
                startButton.Visibility = Visibility.Visible;
            }

            base.OnNavigatedFrom(args);
        }

        void OnStartButtonClick(object sender, RoutedEventArgs args)
        {
            if (captureSource != null && 
                (CaptureDeviceConfiguration.AllowedDeviceAccess ||
                 CaptureDeviceConfiguration.RequestDeviceAccess()))
            {
                captureSource.Start();
                startButton.Visibility = Visibility.Collapsed;
            }
        }

        void OnAppBarCaptureButtonClick(object sender, EventArgs args)
        {
            saveStoryboard.Begin();
            WriteableBitmap captureBitmap = new WriteableBitmap(imgPanel, null);
            DateTime dt = DateTime.Now;
            string filename = String.Format("kaleidovideo-{0:D2}-{1:D2}-{2:D2}-{3:D2}-{4:D2}-{5:D2}",
                                            dt.Year % 100, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
            MemoryStream memoryStream = new MemoryStream();
            captureBitmap.SaveJpeg(memoryStream, captureBitmap.PixelWidth, captureBitmap.PixelHeight, 0, 80);
            memoryStream.Position = 0;
            MediaLibrary mediaLibrary = new MediaLibrary();
            mediaLibrary.SavePicture(filename, memoryStream);
        }
    }
}
