﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;

namespace StraightVideoSink
{
    public partial class MainPage : PhoneApplicationPage
    {
        CaptureSource captureSource;

        public MainPage()
        {
            InitializeComponent();

            VideoCaptureDevice videoCaptureDevice = CaptureDeviceConfiguration.GetDefaultVideoCaptureDevice();

            if (videoCaptureDevice != null)
            {
                videoCaptureDevice.DesiredFormat = new VideoFormat(PixelFormatType.Format8bppGrayscale, 0, 0, 0);
            }

            captureSource = new CaptureSource
            {
                VideoCaptureDevice = videoCaptureDevice
            };

            SimpleVideoSink videoSink = new SimpleVideoSink((WriteableBitmap bitmap) =>
            {
                img.Source = bitmap;
            })
            {
                CaptureSource = captureSource,
                AllocationMode = SampleAllocationMode.ReuseBuffer
            };
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            if (captureSource != null && CaptureDeviceConfiguration.AllowedDeviceAccess)
            {
                captureSource.Start();
                startButton.Visibility = Visibility.Collapsed;
            }
            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            if (captureSource != null && captureSource.State == CaptureState.Started)
            {
                captureSource.Stop();
                startButton.Visibility = Visibility.Visible;
            }
            base.OnNavigatedFrom(args);
        }

        void OnStartButtonClick(object sender, RoutedEventArgs args)
        {
            if (captureSource != null && 
                (CaptureDeviceConfiguration.AllowedDeviceAccess ||
                 CaptureDeviceConfiguration.RequestDeviceAccess()))
            {
                captureSource.Start();
                startButton.Visibility = Visibility.Collapsed;
            }
        }

        void OnAppBarToggleButtonClick(object sender, EventArgs args)
        {
            if (captureSource != null && captureSource.VideoCaptureDevice != null)
            {
                captureSource.Stop();

                if (captureSource.VideoCaptureDevice.DesiredFormat.PixelFormat == PixelFormatType.Format8bppGrayscale)
                    captureSource.VideoCaptureDevice.DesiredFormat = new VideoFormat(PixelFormatType.Format32bppArgb, 0, 0, 0);
                else
                    captureSource.VideoCaptureDevice.DesiredFormat = new VideoFormat(PixelFormatType.Format8bppGrayscale, 0, 0, 0);

                captureSource.Start();
            }
        }
    }
}