﻿using System;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace StraightVideoSink
{
    public class SimpleVideoSink : VideoSink
    {
        VideoFormat videoFormat;
        WriteableBitmap writeableBitmap;
        Action<WriteableBitmap> action;

        public SimpleVideoSink(Action<WriteableBitmap> action)
        {
            this.action = action;
        }

        protected override void OnCaptureStarted() { }

        protected override void OnCaptureStopped() { }

        protected override void OnFormatChange(VideoFormat videoFormat)
        {
            this.videoFormat = videoFormat;

            System.Windows.Deployment.Current.Dispatcher.BeginInvoke(() =>
            {
                writeableBitmap = new WriteableBitmap(videoFormat.PixelWidth, videoFormat.PixelHeight);
                action(writeableBitmap);
            });
        }

        protected override void OnSample(long sampleTimeInHundredNanoseconds, long frameDurationInHundredNanoseconds, byte[] sampleData)
        {
            if (writeableBitmap == null)
                return;

            int baseIndex = 0;

            for (int row = 0; row < writeableBitmap.PixelHeight; row++)
            {
                for (int col = 0; col < writeableBitmap.PixelWidth; col++)
                {
                    int pixel = 0;

                    if (videoFormat.PixelFormat == PixelFormatType.Format8bppGrayscale)
                    {
                        byte grayShade = sampleData[baseIndex + col];
                        pixel = (int)grayShade | (grayShade << 8) | 
                                    (grayShade << 16) | (0xFF << 24);
                    }
                    else
                    {
                        int index = baseIndex + 4 * col;
                        pixel = (int)sampleData[index + 0] | (sampleData[index + 1] << 8) | 
                                (sampleData[index + 2] << 16) | (sampleData[index + 3] << 24);
                    }
                    writeableBitmap.Pixels[row * writeableBitmap.PixelWidth + col] = pixel;
                }
                baseIndex += videoFormat.Stride;
            }
            writeableBitmap.Dispatcher.BeginInvoke(() =>
            {
                writeableBitmap.Invalidate();
            });
        }
    }
}
